pro MSP_BDP_RAW1_Examiner, Event, input_pathname_list, jump_coadds, output_struct
;
;   reads a bunch of RAW1 files and determines the mean counts (and stddev) for
;   each quadrant, putting results in a file for later plotting
;
;   DPC     20050906    initial code
;           20050916    changed program name (from "first_read..." to "RAW1_read..."),
;                       modified to allow calling by batch.pro
;           20050925    fixed input errors
;           20050925    reduced size of examination region from 201x101 to 61x61
;           20060624    copied from RAW1_read_examination - for use on RAW1 files only
;		    20071126	copied from RAW1_examiner.pro. Modified to read from list
;							of pathnames and write results to summary structure
;			20130920	added jump_coadds array to bring in the coadding level for
;							each image
;
;	note: does not work for subarrays yet
;-----------------------------------------------------------------------------
;
WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
s_ptr = PTR_struct.PTR_summary
;
;	set LEDs to yellow
;
colors = strarr(25)
colors[*] = "yellow"
WCT_LED_stack, (*s_ptr).P4_LED_wid[0], colors
;
;	determine number of files on input
;
nfiles = N_ELEMENTS(input_pathname_list)
;
;	and LED scaling
;
LED_scale = float(nfiles) / 25.0d0
if(LED_scale lt 1.0) then LED_scale = 1.0
last_LED = 0
;
;	and create output structure
;
out_struct = {filenum: 0L, mean_even: fltarr(4), s_mean_even: fltarr(4), $
			  mean_odd: fltarr(4), s_mean_odd: fltarr(4) }
output_struct = REPLICATE(out_struct, nfiles)
;
subarray_odd = fltarr(61*61)
subarray_even = fltarr(61*61)
;
;	note: must treat even, odd rows separately for ALADDIN III detector
;
mean_even = fltarr(4)
stddev_even = fltarr(4)
mean_odd = fltarr(4)
stddev_odd = fltarr(4)
;
for ifile = 0, nfiles-1 do begin
	;
	;	verify that file exists before opening
	;
	test = FILE_INFO(input_pathname_list[ifile])
	if( NOT(test.exists) ) then begin
		MSP_BDP_print_text, Event, "Could not find file = "+input_pathname_list[ifile]
		output_struct[ifile].filenum = -1
    	output_struct[ifile].mean_even[*] = 0.0
    	output_struct[ifile].s_mean_even[*] = 0.0
    	output_struct[ifile].mean_odd[*] = 0.0
    	output_struct[ifile].s_mean_odd[*] = 0.0
		goto, loop_end
	endif
	;
	file_size = test.size
	;
	WIDGET_CONTROL, /HOURGLASS
	;
    image = read_raw1(input_pathname_list[ifile])	; returns longword array
    ;
    if(jump_coadds[ifile] ne 1) then begin			; scale by coadds, if present
    	image = long(float(image)/float(jump_coadds[ifile]))
    endif
    ;
    ;   evaluate each quadrant's central properties
    ;
    for ic = 0, 1 do begin
    	;
        ic0 = 220 + ic * 512
        ic1 = 280 + ic * 512
        ;
        for ir = 0, 1 do begin
        	;
           	ir0 = 190 + ir * 512
           	ir1 = 311 + ir * 512
           	;
           	for irow = 0, 60 do begin
           		;
                ii = irow*61
                subarray_even[ii:ii+60] = REFORM(image[ic0:ic1,ir0+2*irow])
                subarray_odd[ii:ii+60] = REFORM(image[ic0:ic1,ir0+2*irow+1])
                ;
           	endfor
           	;
           	;	remove zeros, as these have been fixed by the col clock business in P3
           	;
           	ind_even = WHERE(subarray_even ne 0, n_even)
           	ind_odd = WHERE(subarray_odd ne 0, n_odd)
           	if(n_even gt 1) then begin
           		zz_even = median_filtered_mean(subarray_even[ind_even])
           	endif else begin
           		zz_even = [0, 1000]
           	endelse
           	if(n_odd gt 1) then begin
           		zz_odd = median_filtered_mean(subarray_odd[ind_odd])
           	endif else begin
           		zz_odd = [0, 1000]
           	endelse
           	;
           	iz = ic + 2 * ir
           	mean_even[iz] = zz_even[0]
           	stddev_even[iz] = zz_even[1]
           	mean_odd[iz] = zz_odd[0]
           	stddev_odd[iz] = zz_odd[1]
        endfor
        this_LED = 0 > (24 < ROUND(float(ifile) / LED_scale))
		colors[this_LED:this_LED] = "blue"
		WCT_LED_stack, (*s_ptr).P4_LED_wid[0], colors
    endfor
	this_LED = 0 > (24 < ROUND(float(ifile) / LED_scale))
	colors[this_LED:this_LED] = "violet"
	WCT_LED_stack, (*s_ptr).P4_LED_wid[0], colors
    ;
    ;   strip off frame number from filename
    ;
    frame = FILE_BASENAME(input_pathname_list[ifile])
    ;
    ;   3 or 4 digits of serial number after "raw1_"
    ;
    len = STRLEN(frame)
    frame_no = -1
    if(len eq 8) then begin
        reads, frame, format='(5x,i3)',frame_no
    endif else begin
        if(len eq 14) then begin
           ;
           ; has root of type "lin_#####_RAW1"
           ;
           reads, frame, format='(4x,i5)',frame_no
        endif else begin
            reads, frame, format='(5x,i4)',frame_no
        endelse
    endelse
    if(frame_no eq -1) then begin
    	print, 'ERROR - could not create frame_no'
    	stop
    endif
    ;
    ;	add to output structure
    ;
    output_struct[ifile].filenum = frame_no
    output_struct[ifile].mean_even[*] = mean_even[*]
    output_struct[ifile].s_mean_even[*] = stddev_even[*]
    output_struct[ifile].mean_odd[*] = mean_odd[*]
    output_struct[ifile].s_mean_odd[*] = stddev_odd[*]
    ;
    ;printf, out_lun, format="(a,1h;,i4,1h;,8(F8.1,1h;,F7.2,1h;))",filename,frame_no,mean_even[0],stddev_even[0],mean_odd[0],stddev_odd[0],$
    ;         mean_even[1],stddev_even[1],mean_odd[1],stddev_odd[1],$
    ;            mean_even[2],stddev_even[2],mean_odd[2],stddev_odd[2],$
    ;            mean_even[3],stddev_even[3],mean_odd[3],stddev_odd[3]
    ;print, filename
    ;
    loop_end:
    ;
    this_LED = 0 > (24 < ROUND(float(ifile) / LED_scale))
	colors[this_LED:this_LED] = "green"
	WCT_LED_stack, (*s_ptr).P4_LED_wid[0], colors
    ;
endfor
;
end
