;*******************************PRELIMINARY FUNCTIONS*******************************
;The following Set Operations are taken from the website: http://www.idlcoyote.com/

FUNCTION SetUnion, a, b
;Input
;-a: A set of elements of any single number type
;-b: A different set of number elements

;Output
;-union: The mathematical union of set a and b)

superset = [a, b]
union = superset[Uniq(superset, Sort(superset))]
RETURN, union
END

FUNCTION SetIntersection, a, b
;Input
;-a: A set of elements of any single type
;-b: A different set of number elements

;Output
;-r + minab: The mathematical intersection of set a and b
;-null: a -1 output if there are no elements belonging to both set a and b

;Caution: This procedure only works for sets with all unique elements
mina = Min(a, Max=maxa)
minb = Min(b, Max=maxb)
minab = mina > minb
maxab = maxa < maxb

; If the set ranges don't intersect, then result = NULL.
IF ((maxa LT minab) AND (minb GT maxab)) OR $
   ((maxb LT minab) AND (mina GT maxab)) THEN RETURN, -1

r = Where((Histogram(a, Min=minab, Max=maxab) NE 0) AND  $
          (Histogram(b, Min=minab, Max=maxab) NE 0), count)

IF count EQ 0 THEN RETURN, -1 ELSE RETURN, r + minab
END

FUNCTION SetDifference, a, b
;Input
;-a: A set of elements of any single type
;-b: A different set of number elements

;Output
;-r + mina: a set of elements belonging to set a but not belonging to set b
;-a: If no elements are held in common, then set a is returned as entirely uniqe

mina = Min(a, Max=maxa)
minb = Min(b, Max=maxb)
IF (minb GT maxa) OR (maxb LT mina) THEN RETURN, a ;No intersection...
r = Where((Histogram(a, Min=mina, Max=maxa) NE 0) AND $
          (Histogram(b, Min=mina, Max=maxa) EQ 0), count)
IF count eq 0 THEN RETURN, -1 ELSE RETURN, r + mina
END

;***********************************MAIN PROGRAM***********************************
;The following function was written by Jordan Montgomery, BU, October 22nd, 2012

FUNCTION LOWER_LEFT_QUADRANT_REPAIR, img_ptr, init_bad

;Input
;-img_ptr: Pointer to an array containing a bad lower left quadrant in need of repair.
;          This array will be repaired in heap
;-init_bad: Boolean 1 or 0 declaring the input image as an initial failure (1) or a subsequent failure (0)

;Output
;-status: Boolean 1 or 0 declaring the success or failure, respectively, of repairing the damaged quadrant

LLQ        = (*img_ptr)[0:511,0:512]									;Cut out the lower left quadrant (LLQ)
LRQ        = (*img_ptr)[512:1023,0:512]									;Cut out the lower right quadrant (LRQ)
repair_map = INTARR(512,513)											;Create an map of which pixels need to be fixed
start_row = 0															;At what row to begin examining the image
end_row   = 512															;Top of the LLQ
tot_nlow  = 0															;Initalize a low-pixel counter
IF init_bad eq 1 THEN BEGIN
;************************PROCESSING FOR INITIAL FAILED IMAGE************************
	row      = 12														;Use a higher start row for inital bad image tests
	good_bin = 1
	all_bad  = 0
	WHILE good_bin AND (row LE end_row) DO BEGIN
		LLQ_row = LLQ[*,row]											;Use the "bad row" to compute the background level
		tol     = (MEDIAN_FILTERED_MEAN(LRQ[*,row]))[1]					;Find the typical dispersion in values from the LRQ
		LLQ_med = MEDIAN(LLQ_row)										;Find the median value of the LLQ
		off_med = ABS(LLQ_row - LLQ_med)								;Calculate the value offset from the median
		LLQ_row1= LLQ_row[WHERE(off_med LT 4.0*tol)]						;Eliminate values far (4-sigma) from the median
		zz      = MEDIAN_FILTERED_MEAN(LLQ_row1)						;Calculate a median filtered mean of remaining values
		ind_low = WHERE(ABS(LLQ_row - zz[0] + 256.0) LT 4.0*zz[1], nLow);Find the bad pixels (the 256 bit)

		tot_nlow += nlow												;Count the total number of low pixels
		IF ((row - start_row + 1) MOD 5) EQ 0 THEN BEGIN				;After collecting stats on five rows
			IF tot_nlow GE 120 THEN good_bin = 0						;Determine if the bin contains bad rows
			tot_nlow = 0												;Reset the low-pixel counter
		ENDIF
		row++															;Increment the row number

		IF ~good_bin THEN BEGIN
			IF (row - 5 - start_row) LT 5 THEN all_bad = 1				;Test if the first bin was bad
			IF all_bad THEN BEGIN
				repair_map[0:511,0:512] = 2								;If all bad, set whole quadrant to "trash" value
			ENDIF ELSE BEGIN
				repair_map[0:511, row-5:512] = 2						;Otherwise set only bad section of quadrant to "trash" value
			ENDELSE
		ENDIF
	ENDWHILE
ENDIF ELSE BEGIN
;***********************PROCESSING FOR SUBSEQUENT FAILED IMAGE***********************
	tot_bad = 0
	FOR row = start_row, end_row DO BEGIN
		LLQ_row  = LLQ[*,row]											;Use the "bad row" to compute the background level
		tol      = (MEDIAN_FILTERED_MEAN(LRQ[*,row]))[1]				;Find the typical dispersion in values from the LRQ
		LLQ_med  = MEDIAN(LLQ_row)										;Find the median value of the LLQ
		off_med  = ABS(LLQ_row - LLQ_med)								;Calculate the value offset from the median
		LLQ_row1 = LLQ_row[WHERE(off_med LT 4.0*tol)]						;Eliminate values far (4-sigma) from the median
		zz       = MEDIAN_FILTERED_MEAN(LLQ_row1)						;Calculate a median filtered mean of remaining values

		ind_high = WHERE(ABS(LLQ_row - zz[0] - 256.0) LT 3.0*zz[1], nHigh)	;Find the bad pixels (the 256 bit)
		ind_low  = WHERE(ABS(LLQ_row - zz[0] + 256.0) LT 3.0*zz[1], nLow)

		IF nLow  GT 0 THEN repair_map[ind_low, row]  = -1				;Mark the bad pixels
		IF nHigh GT 0 THEN repair_map[ind_high, row] = +1
		tot_bad +=(nHigh + nLow)
	ENDFOR
ENDELSE
;********************************DETECTION STATISTICS********************************
low_pix  = WHERE(repair_map EQ -1,nLow)
high_pix = WHERE(repair_map EQ +1,nHigh)
trsh_pix = WHERE(repair_map EQ +2,nTrsh)
IF nLow  GT 0 THEN LLQ[low_pix]  +=256.0
IF nHigh GT 0 THEN LLQ[high_pix] -=256.0
IF nTrsh GT 0 THEN LLQ[trsh_pix]  = -1E6

(*img_ptr)[0:511,0:512] = LLQ

;**How to determine "fix status"???**
status = 1
;**How to determine "fix status"???**

RETURN, status

END